# $Header:   //ARCHIVE/Archive_NLS/scripts/CollectTBSMServiceDefs.ksv   1.1   Jun 12 2001 13:25:36   BRENT  $
#
# CollectTBSMServiceDefs.ksh
#
# Module Description
#   Collects the list of TBSM-related services from a specified set of NT
#   servers and populates the TBSMServiceDef table in the Object database.
#   TBSMServiceDef is used by the TBSM_Diagnostics stored procedure.

typeset -i bApply=1
typeset -i bTrusted=0
typeset -i bReset=0
typeset -i bDeleteOldServices=0
typeset -i bSetDesiredState=0

trap "rm -f CollectTBSMServiceDefs.$$; exit" ERR EXIT

usage	()
{
  print ""
  print "Usage: "$0" <options>... <hosts>...."
  print ""
  print "  Options are:"
  print "  -U<db_user>       Database user (default=$db_user)"
  print "  -P<db_password>   Database password"
  print "  -U<db_server>     Database user (default=$db_server)"
  print "  -r                Reset current list of services in database"
  print "  -c                Use current state of services as desired state. The"
  print "                    default is to set the desired state of all discovered"
  print "                    services to 'running'"
  print "  -d                Delete any service definition records for services"
  print "                    that were not rediscovered during this collection"
  print "  -n                Do not apply the changes to the database. When this"
  print "                    option is used the script will prepare a file called"
  print "                    CollectTBSMServiceDefs.sql in the current directory."
  print "                    This file can then be applied to the database if"
  print "                    desired."
  print "  -E                Use trusted connection. Use this instead of -U and -P"
  print ""
  print "  Example:"
  print "    The following command will collect service definitions from a"
  print "    specified set of TBSM NT servers and populate them in the local"
  print "    database using a trusted connection."
  print "    sh $0 -E TBSMDB TBSMAPP TBSMPROP TBSMSNA TBSMEVENT TBSMHIST"
}

# Get Database Server, User, Password from Registry
eval $(regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}')

# Process input parameters
while getopts "U:S:P:Ecdnr?" opt; do
	case $opt in
  U)  db_user=$OPTARG;;
  P)  db_pasword=$OPTARG;;
  S)  db_server=$OPTARG;;
  E)  bTrusted=1;;
  r)  bReset=1;;
  d)  bDeleteOldServices=1;;
  c)  bSetDesiredState=1;;
	\?) usage
		  exit 16
		  ;;
	esac
done
shift $(($OPTIND - 1))

# Check for required params
if [ -z "$1" ]; then
  print >&2 "No hosts specified!"
  print >&2 ""
  usage>&2
  exit 1
fi

# Collect TBSM services from hosts
rm -f CollectTBSMServiceDefs.$$
while [ ! -z "$1" ]; do
  host=$1
  sclist $host | egrep "BSM|SQL|SnaBase|SnaServer|RemoteKill|W3SVC|[^A-z]Schedule[^A-z]" | awk -v host=$host -v bSetDesiredState=$bSetDesiredState '{
    if (bSetDesiredState) {
      state=substr($0,1,17);
      sub("[ ]*$","",state);
      if (state=="start pending") state="running";
      else if (state=="paused") state="running";
      else if (state=="stop pending") state="stopped";
    }
    else
      state="running";
    svc=substr($0,18,33);
    sub("[ ]*$","",svc);
    print "INSERT INTO ##TBSMServiceDef(hostname,servicename,desiredstate) VALUES ( '\''" host "'\'', '\''" svc "'\'', '\''" state "'\'' )"
  }' >> CollectTBSMServiceDefs.$$
  shift
done

SQL=osql

if [ $bTrusted -eq 1 ]; then
  SQL_CMD_LINE="$SQL -S$db_server -E -n -w3000 -dObject";
else
  SQL_CMD_LINE="$SQL -S$db_server -U$db_user -P$db_password -n -w3000 -dObject";
fi

rm -f CollectTBSMServiceDefs.sql

print "SET NOCOUNT ON" >> CollectTBSMServiceDefs.sql
print "USE Object" >> CollectTBSMServiceDefs.sql
print "" >> CollectTBSMServiceDefs.sql
print -R "-- Create temporary table for all discovered services" >> CollectTBSMServiceDefs.sql
print "IF EXISTS (SELECT * FROM tempdb..sysobjects WHERE name='##TBSMServiceDef') EXEC(\"DROP TABLE ##TBSMServiceDef\")" >> CollectTBSMServiceDefs.sql
print "CREATE TABLE ##TBSMServiceDef ( hostname NVARCHAR(255) NOT NULL, servicename NVARCHAR(40) NOT NULL, desiredstate NVARCHAR(16) NULL )" >> CollectTBSMServiceDefs.sql
print "GO" >> CollectTBSMServiceDefs.sql
print "" >> CollectTBSMServiceDefs.sql
print "BEGIN TRANSACTION" >> CollectTBSMServiceDefs.sql
print "" >> CollectTBSMServiceDefs.sql

cat CollectTBSMServiceDefs.$$ >> CollectTBSMServiceDefs.sql

if [ $bReset = 1 ]; then
  print "" >> CollectTBSMServiceDefs.sql
  print -R "-- Remove all old definitions since -r was specified" >> CollectTBSMServiceDefs.sql
  print "DELETE FROM TBSMServiceState" >> CollectTBSMServiceDefs.sql
  print "DELETE FROM TBSMServiceDef" >> CollectTBSMServiceDefs.sql
fi

print "" >> CollectTBSMServiceDefs.sql
print -R "-- Create new service definitions" >> CollectTBSMServiceDefs.sql
print "INSERT INTO TBSMServiceDef(hostname,servicename,desiredstate)" >> CollectTBSMServiceDefs.sql
print "  SELECT * FROM ##TBSMServiceDef AS newsvc WHERE NOT EXISTS" >> CollectTBSMServiceDefs.sql
print "    (SELECT * FROM TBSMServiceDef AS oldsvc WHERE oldsvc.hostname=newsvc.hostname AND oldsvc.servicename=newsvc.servicename)" >> CollectTBSMServiceDefs.sql

if [ $bSetDesiredState = 1 ]; then
  print "" >> CollectTBSMServiceDefs.sql
  print -R "-- Update desiredstate of existing services since -c was specified" >> CollectTBSMServiceDefs.sql
  print "UPDATE oldsvc SET desiredstate=newsvc.desiredstate" >> CollectTBSMServiceDefs.sql
  print "FROM TBSMServiceDef AS oldsvc" >> CollectTBSMServiceDefs.sql
  print "  JOIN ##TBSMServiceDef AS newsvc ON" >> CollectTBSMServiceDefs.sql
  print "    newsvc.hostname=oldsvc.hostname" >> CollectTBSMServiceDefs.sql
  print "    AND newsvc.servicename=oldsvc.servicename" >> CollectTBSMServiceDefs.sql
  print "    AND newsvc.desiredstate<>oldsvc.desiredstate" >> CollectTBSMServiceDefs.sql
fi

if [ $bDeleteOldServices = 1 ]; then
  print "" >> CollectTBSMServiceDefs.sql
  print -R "-- Remove old service definitions since -d was specified" >> CollectTBSMServiceDefs.sql
  print "DELETE FROM oldsvc FROM TBSMServiceState AS oldsvc LEFT OUTER JOIN ##TBSMServiceDef AS newsvc ON" >> CollectTBSMServiceDefs.sql
  print "    newsvc.hostname=oldsvc.hostname" >> CollectTBSMServiceDefs.sql
  print "    AND newsvc.servicename=oldsvc.servicename" >> CollectTBSMServiceDefs.sql
  print "  WHERE newsvc.hostname IS NULL" >> CollectTBSMServiceDefs.sql
  print "DELETE FROM oldsvc FROM TBSMServiceDef AS oldsvc LEFT OUTER JOIN ##TBSMServiceDef AS newsvc ON" >> CollectTBSMServiceDefs.sql
  print "    newsvc.hostname=oldsvc.hostname" >> CollectTBSMServiceDefs.sql
  print "    AND newsvc.servicename=oldsvc.servicename" >> CollectTBSMServiceDefs.sql
  print "  WHERE newsvc.hostname IS NULL" >> CollectTBSMServiceDefs.sql
fi

print "" >> CollectTBSMServiceDefs.sql
print "COMMIT TRANSACTION" >> CollectTBSMServiceDefs.sql

if [ $bApply = 1 ]; then
  $SQL_CMD_LINE -iCollectTBSMServiceDefs.sql
  rm -f CollectTBSMServiceDefs.sql
fi
