# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imVersionLookup.ksv   1.0   Jun 18 2001 16:51:38   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imVersionLookup.ksv  $
# $Revision:   1.0  $
# $Date:   Jun 18 2001 16:51:38  $
# $Author:   lin  $
#!/bin/sh
#set -x
#
# imVersionLookup.ksh
#   Script to parse an input string and return a version number from imSnapshotHistory table.
#   The output version number will be printed to stdout, any error message will be printed to stderr.
#   Input string can be in any of the following format
#
#     version in all numeric characters
#       version number specified, if it is within the valid range, will be returned. 
#
#     date in YYYY-MM-DD[ hh:mm:ss] 
#       version number of the the first record that has the date greater or equal to the date specified will 
#         be returned. 
#
#     string in type|name[|build]
#       version number of the the first record that has the type and name matches (and build, if specified) 
#         the input string will be returned.
#

#
# Lookup the installation directory from the registry
#
INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Unknown installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

#
# check for OSQL utility
#
rc=`which osql.exe`; 
rc=$?
if [ $rc -ne 0 ]; then
  print >&2 "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." 
  exit 1
fi

#
# Get Database Server, User, Password from Registry
#
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

#
# set SQLCMD command
#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 -dObject "
  SQLCMD="osql -h-1 -S$db_server -E -n -w3000 -dObject "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 -dObject "
  SQLCMD="osql -h-1 -S$db_server -U$db_user -P$db_password -n -w3000 -dObject "
fi

#
# get the Max snapshot version
#
getMAXversion()
{
  sql_cmd="SET NOCOUNT ON; SELECT MAX(version) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}'
}

#
# get the Min snapshot version
#
getMINversion()
{
  sql_cmd="SET NOCOUNT ON; SELECT MIN(version) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}'
}

#
# get the Max snapshot date
#
getMAXdate()
{
  sql_cmd="SET NOCOUNT ON; SELECT MAX(date) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {date=$0} END {print date}'
}

#
# get the Min snapshot date
#
getMINdate()
{
  sql_cmd="SET NOCOUNT ON; SELECT MIN(date) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {date=$0} END {print date}'
}

#
# default values & local variables
#
HOSTNAME=`hostname`
show_history=0

db_firstver=$(getMINversion)
db_lastver=$(getMAXversion)
db_firstdate=$(getMINdate)
db_lastdate=$(getMAXdate)

usage()
{
  print >&2 ''
  print >&2 'Usage: '$0' -i<input_string> [options...]'
  print >&2 'Output is always sent to stdout.'
  print >&2 ''
  print >&2 '  -S<db_server>       Specifies server name where database resides.  Default=['$db_server'].'
  print >&2 '  -U<db_user>         Specifies database user name.  Default=['$db_user'].'
  print >&2 '  -P<db_password>     Specifies password for database user.  Default=['$db_password'].'
  print >&2 '  -E                  Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print >&2 '  -i<input_string>    requried input string to be mapped to a version number' 
  print >&2 '                      input_string can be in the following format:'
  print >&2 '                        date in YYYY-MM-DD[ hh:mm:ss], within the range of $db_firstdate, $db_lastdate'
  print >&2 '                          example: 2001-06-22 or "2001-06-29 12:23:34" '
  print >&2 '                        version in digit, within the range of $db_firstver, $db_lastver '
  print >&2 '                          example: 2 or 39 '
  print >&2 '                        string in <type>|<name>[|<build>]'
  print >&2 '                          example: "Regular|TBSM 1.5" or "Patch|Patch 1.5.22|V3.2.1"  '
  print >&2 ''
  print >&2 '  -h                  help'
  print >&2 '  -?                  help'
  print >&2 ''
}

while getopts "S:U:P:E:i:h?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  i)  inputString=$OPTARG;;

  h) usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

#
# inputString is required
#
if [ -z "$inputString" ]; then
  print >&2 "Missing required input string!"
  exit 2
fi

# trim leading and trailing spaces
inputString=$(echo $inputString | awk ' { sub("^  *", "", $0); sub("  *$", "", $0); print $0 } ')

#---------------------------------------------------------------------------------------------------------
# look for version number in the inputString
#---------------------------------------------------------------------------------------------------------
version=$(echo $inputString | awk '/^[0-9][0-9]*$/ { print $0 }')

if [ ! -z "$version" ]; then
  # found a valid version number, check if it is withing range
  if [ "$version" -gt "$db_lastver" ] || [ "$version" -lt "$db_firstver" ]; then
    print >&2 "Unknown version number $version entered."
    print >&2 "  It should be a number in between $db_firstver and $db_lastver!"
    exit 3
  else
    print $version
    exit 0
  fi
fi

#---------------------------------------------------------------------------------------------------------
# look for date in the inputString
# return the first snapshot version that has a date greater or equal to the date specified
#---------------------------------------------------------------------------------------------------------
date=$(echo $inputString | awk '/^[1-2][0-9]{3}-([0]*[0-9]|[1][0-2])-[0-3]*[0-9]( ([0-1]*[0-9]|[2][0-3]):[0-5]*[0-9]:[0-5]*[0-9])*/ { print $0 } ')

if [ ! -z "$date" ]; then
  # the input string is in date format
  # find the version number for the date >= then the inputString in the imSnapshotHistory table
  sql_cmd="SET NOCOUNT ON; SELECT version FROM imSnapshotHistory WHERE  date >= '$date' "
  version=$($SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ { print $1; exit }')

  if [ -z "$version" ]; then
    print >&2 "Unknown date $date entered."
    print >&2 "  It should be a date in between $db_firstdate and $db_lastdate!"
    exit 3 
  else
    print $version
    exit 0
  fi
fi

#---------------------------------------------------------------------------------------------------------
# look for <type>|<name>[|<build>]
# exact match is performed
#---------------------------------------------------------------------------------------------------------
type=$(echo $inputString | awk -F"|" ' { print $1 } ') 
name=$(echo $inputString | awk -F"|" ' { print $2 } ') 
build=$(echo $inputString | awk -F"|" ' { print $3 } ') 

# type and name are required 
if [ -z "$type" ] || [ -z "$name" ]; then 
  print >&2 "Unknown input \"$inputString\" entered, missing <Type>|<Name>[|<build>]."
  exit 3
fi

if [ -z "$build" ]; then 
  # build is not specified
  sql_cmd="SET NOCOUNT ON; 
    SELECT version FROM imSnapshotHistory WHERE  type = '$type' AND name = '$name'
    ORDER BY version
  "
else
  sql_cmd="SET NOCOUNT ON; 
    SELECT version FROM imSnapshotHistory WHERE  type = '$type%' AND name = '$name'
    AND build LIKE '$build%' ORDER BY version 
  "
fi

version=$($SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ { print $1; exit }')

#
# give user the available choices if input is not correct
#
if [ -z "$version" ]; then
  print >&2 "Unknown <type>|<name>[|<build>] $inputString entered."
  print >&2 "  Please specify one of the following <type>|<name>[|<build>] or the version number it points to:"
  sql_cmd="SET NOCOUNT ON; 
    SELECT type+'|'+name+'|'+build+'|', version FROM imSnapshotHistory
    WHERE state = 'Started'
    ORDER BY version
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" | while read outputString; do
    if [ -z "$outputString" ]; then
      continue
    fi

    type=$(echo $outputString | awk -F"|" ' { print $1 } ') 
    name=$(echo $outputString | awk -F"|" ' { print $2 } ') 
    build=$(echo $outputString | awk -F"|" ' { print $3 } ') 

    if [ "$build" == " " ]; then
      build= 
    fi
    ver=$(echo $outputString | awk -F"|" ' { print $4 } ') 
    print -n >&2 "    $type|$name"
    if [ -n "$build" ]; then
      print -n >&2 "|$build --> version $ver"
    else
      print -n >&2 " --> version $ver"
    fi
    print >&2
  done

  exit 3
else
  print $version
  exit 0
fi

