-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/DEFECT_76617.sqv   1.0   Jun 05 2001 16:20:18   BRENT  $
--
-- DEFECT_76617.sql
--
-- Module Description
--  Fixes defect 76617.
--
--  The _State column of StateType_A has always been 16 characters
--  wide but this is not sufficient for some RODM/SNA states such
--  as "positive user-defined (136)".  They would get truncated and
--  make drop-down lists look very confusing because several states
--  could have the same truncated name.
--
--  We will extend the width of the _State column to 40 characters and
--  fix those rows that were truncated.

USE Object
GO


---------------------------------------------------------------------------------------------------
-- UpgradeStateType
PRINT 'Creating procedure UpgradeStateType'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeStateType' AND type = 'P')
  DROP PROCEDURE UpgradeStateType
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/SQL/DEFECT_76617.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jun 05 2001 16:20:18  $
-- $Author:   BRENT  $
CREATE PROCEDURE UpgradeStateType
AS

  IF (SELECT prec FROM syscolumns WHERE id = OBJECT_ID('StateType_A_LOCAL') AND name = '_State') = 40
  BEGIN
    PRINT 'StateType_A._State has already been upgraded'
    RETURN
  END

  PRINT 'Extending the width of the _State column in StateType_A'


  BEGIN TRANSACTION UpgradeStateType

  -- we'll rebuild the views later

  EXEC("DROP VIEW StateType_V")
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not drop view StateType_V', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  EXEC("DROP VIEW StateType_A")
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not drop view StateType_A', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  EXEC("ALTER TABLE StateType_A_LOCAL ALTER COLUMN _State NVARCHAR(40)")
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not alter StateType_A_LOCAL', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  -- update the _State column where ever it appears to be truncated

  UPDATE StateType_A_LOCAL SET _State = _Desc
  WHERE _State <> _Desc AND LEN(_State) = 16 AND SUBSTRING(_Desc,1,16) = _State
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not update StateType_A_LOCAL._State', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  -- recreate the views

  EXEC("
  CREATE VIEW StateType_A
  AS
    SELECT
      b.id,
      l._State,
      l._Desc,
      b._Order,
      b._Flags
    FROM StateType_A_BASE AS b JOIN StateType_A_LOCAL AS l ON
      b.id = l.id
    WHERE l.langid=@@LANGID
  ")
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not create StateType_A view', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  EXEC("
  CREATE VIEW StateType_V AS
    SELECT o.id, name, revision, nchild, description, o.ctime, mtime, userid, deleted,
      _Desc,
      _Flags,
      _Order,
      _State
  FROM
    StateType_C o,
    StateType_A a
  WHERE
    o.id = a.id
  ")
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('Could not alter StateType_V view', 3, 1)
    ROLLBACK TRANSACTION UpgradeStateType
    GOTO Reset
  END

  COMMIT TRANSACTION UpgradeStateType

Reset:

  PRINT 'StateType_A_LOCAL._State successfully upgraded'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeStateType' AND type = 'P')
BEGIN
  -- set compatibility level to 7.0 so we can do the ALTER COLUMN -- must do it outside the transaction
  EXEC sp_dbcmptlevel Object, 70
  EXEC UpgradeStateType
  -- set compatibility level back to 6.5
  EXEC sp_dbcmptlevel Object, 65
END
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'UpgradeStateType' AND type = 'P')
  DROP PROCEDURE UpgradeStateType
GO

